package com.yanzuoguang.cloud.aop;

import com.yanzuoguang.cloud.CloudContans;
import com.yanzuoguang.cloud.service.TokenServiceCall;
import com.yanzuoguang.util.base.ObjectHelper;
import com.yanzuoguang.util.contants.ResultConstants;
import com.yanzuoguang.util.exception.ExceptionHelper;
import com.yanzuoguang.util.helper.JsonHelper;
import com.yanzuoguang.util.helper.StringHelper;
import com.yanzuoguang.util.log.Log;
import com.yanzuoguang.util.vo.LogVo;
import com.yanzuoguang.util.vo.Ref;
import com.yanzuoguang.util.vo.ResponseResult;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.lang.reflect.Method;
import java.lang.reflect.Type;

/**
 * LogsAspect（接口请求日志切面）
 *
 * @author: Kang
 * @time: 2018年04月25日 11:43
 */
@Aspect
@Component
public class WebAspect extends BaseRequestAspect {

    private static final Logger logger = LoggerFactory.getLogger(BaseRequestAspect.class);

    @Autowired
    private TokenServiceCall tokenServiceCall;

    /**
     * exec aop point aspect
     */
    @Pointcut("execution(* *..web..*Controller.*(..))")
    public void webAspect() {
    }

    /**
     * 执行环形切面
     *
     * @param joinPoint
     * @return
     */
    @Around(value = "webAspect()")
    public Object requestWebAround(ProceedingJoinPoint joinPoint) throws Throwable {
        Log.threadBegin();
        // 用户数据库记录
        ResponseResult responseResult = null;
        long start = System.currentTimeMillis();
        String name = joinPoint.getSignature().getName();
        Ref<Boolean> flag = new Ref<>(false);
        try {
            logger.info("[ {} ] request params is {}", name, joinPoint.getArgs());
            tokenServiceCall.tokenInit();

            Object result = executeMethod(joinPoint, name);
            if (result instanceof ResponseResult) {
                responseResult = (ResponseResult) result;
            } else {
                responseResult = ResponseResult.result(result);
            }

            long end = System.currentTimeMillis();
            logger.info("[ {} ] time ({})ms, result is {}", name, (end - start), responseResult);
            tokenFinish(flag);
            return result;
        } catch (Exception e) {
            long end = System.currentTimeMillis();
            logger.error("[ {} ] time ({})ms,  is error {}", name, (end - start), e);
            responseResult = ExceptionHelper.getError(e);
            if (getReturnType(joinPoint).getTypeName().indexOf(ResponseResult.class.getName()) > -1) {
                return responseResult;
            } else {
                throw e;
            }
        } finally {
            tokenFinish(flag);
            Log.threadCommit();
            saveInterLogs(joinPoint, responseResult);
        }
    }

    /**
     * 执行结束函数
     *
     * @param flag 结束函数
     * @return 结束函数
     */
    private void tokenFinish(Ref<Boolean> flag) {
        if (flag.value == true) {
            return;
        }
        flag.value = true;
        tokenServiceCall.tokenFinish();
    }

    /**
     * 获取返回的至类型
     *
     * @param joinPoint
     * @return
     * @throws NoSuchMethodException
     */
    private Type getReturnType(ProceedingJoinPoint joinPoint) {
        //获取返回值类型
        Signature s = joinPoint.getSignature();
        MethodSignature ms = (MethodSignature) s;
        Method m = ms.getMethod();
        Type t = m.getAnnotatedReturnType().getType();
        return t;
    }

    private Object executeMethod(ProceedingJoinPoint joinPoint, String name) throws Throwable {
        boolean dataArgs = joinPoint.getArgs().length != 1
                || joinPoint.getArgs().length == 1 &&
                (joinPoint.getArgs()[0] instanceof ServletResponse || joinPoint.getArgs()[0] instanceof ServletRequest);
        if (dataArgs) {
            return joinPoint.proceed();
        } else {
            // 获取请求编号
            Object firstArgs = joinPoint.getArgs().length > 0 ? joinPoint.getArgs()[0] : null;
            String reqId = ObjectHelper.getString(firstArgs, "reqId");
            if (StringHelper.isEmpty(reqId)) {
                reqId = StringHelper.md5(JsonHelper.serialize(firstArgs));
            } else {
                // 请求编号和公司编号挂钩
                reqId = StringHelper.getId(ObjectHelper.getString(firstArgs, "companyId"), reqId);
            }
            String reqFull = StringHelper.getId(reqId, HttpAspectUtil.getHttpRequestUrl());
            RequestCacheResult req = cacheResult.get(reqFull, new RequestCacheResult(reqFull));
            // 缓存的键值对
            if (req.getResult() == null) {
                synchronized (req) {
                    if (req.getResult() == null) {
                        try {
                            req.setResult(joinPoint.proceed());
                        } catch (Exception ex) {
                            req.setResult(ex);
                        }
                    }
                }
            }
            Object result = req.getResult();
            if (result instanceof Throwable) {
                throw (Throwable) result;
            }
            return result;
        }
    }

    /**
     * deal with inter api logs
     *
     * @param joinPoint      请求参数
     * @param responseResult 返回参数
     */
    private void saveInterLogs(ProceedingJoinPoint joinPoint, ResponseResult responseResult) {
        // 日志请求不记录,防止死循环递归
        if (applicationName.indexOf(CloudContans.LOG_MODULE) >= 0) {
            return;
        }
        // 正常请求不记录
        if (!logAll
                && responseResult != null
                && responseResult.getCode() == ResultConstants.SUCCESS) {
            return;
        }
        LogVo logVo = initLogInterVo(HttpAspectUtil.getHttpRequestUrl(), joinPoint, responseResult);
        addLog(logVo);
    }
}
